//
// Copyright (c) 2008 smdn <smdn@smdn.jp>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;
using Smdn.Net.Imap4.Protocol.DataStructures;

namespace Smdn.Net.Imap4.Client {
  public class ImapMessageThread {
    public ImapMessageThread Parent {
      get; private set;
    }

    public bool IsRoot {
      get { return Parent == null; }
    }

    public int Depth {
      get { return Parent == null ? 0 : Parent.Depth + 1; }
    }

    public ImapMessage Message {
      get {
        if (Parent == null)
          throw new NotSupportedException("can not get message of thread root");
        else
          return message;
      }
    }

    public IReadOnlyList<ImapMessageThread> Children {
      get; private set;
    }

    internal ImapMessageThread(ImapThreadList threadList, IDictionary<long, ImapMessage> messages)
      : this(null, threadList, messages)
    {
    }

    private ImapMessageThread(ImapMessageThread parent,
                              ImapThreadList threadList,
                              IDictionary<long, ImapMessage> messages)
    {
      this.Parent = parent;

      if (parent == null)
        this.message = null;
      else
        this.message = messages[threadList.Number];

      this.Children = threadList.Children
                                .ConvertAll(t => new ImapMessageThread(this, t, messages));
    }

    public void Traverse(Action<ImapMessageThread> visitThread)
    {
      Traverse(visitThread, null);
    }

    public void Traverse(Action<ImapMessageThread> visitInToThread,
                         Action<ImapMessageThread> visitOutFromThread)
    {
      if (visitInToThread != null && !IsRoot)
        visitInToThread(this);

      foreach (var child in Children) {
        child.Traverse(visitInToThread, visitOutFromThread);
      }

      if (visitOutFromThread != null && !IsRoot)
        visitOutFromThread(this);
    }

    private readonly ImapMessage message;
  }
}

