// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Net.Imap4.Protocol.Transmission;
using Smdn.Text;

namespace Smdn.Net.Imap4.Protocol.DataStructures.BodyStructures {
  public sealed class ImapExtendedSinglePartBodyStructure : ImapSinglePartBodyStructure, IImapExtendedBodyStructure {
    public string MD5 { get; }
    public ImapBodyDisposition Disposition { get; }
    public IReadOnlyList<string> Languages { get; }
    public Uri Location { get; }
    public IReadOnlyList<ImapData> Extensions { get; }

    internal ImapExtendedSinglePartBodyStructure(ImapSinglePartBodyStructure baseStructure,
                                                 string md5,
                                                 ImapBodyDisposition disposition,
                                                 IReadOnlyList<string> languages,
                                                 Uri location,
                                                 IReadOnlyList<ImapData> extensions)
      : base(baseStructure)
    {
      MD5           = md5;
      Disposition   = disposition;
      Languages     = languages ??
#if NET45
        ArrayExtensions.Empty<string>();
#else
        Array.Empty<string>();
#endif
      Location      = location;
      Extensions    = extensions ??
#if NET45
        ArrayExtensions.Empty<ImapData>();
#else
        Array.Empty<ImapData>();
#endif
    }

    public override string ToString()
    {
      return StringConversion.ToString(GetType(), new (string, object)[] {
        ("BaseStructure", base.ToString()),
        (nameof(MD5), MD5),
        (nameof(Disposition), Disposition),
        (nameof(Languages), Languages),
        (nameof(Location), Location),
        (nameof(Extensions), Extensions),
      });
    }
  }
}
