// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Text;

namespace Smdn.Net.Imap4.Protocol.DataStructures.BodyStructures {
  public class ImapMessageRfc822BodyStructure : ImapSinglePartBodyStructure {
    /// <summary>The envelope of encapsulated message.</summary>
    public ImapEnvelope Envelope { get; }

    /// <summary>The body structure of encapsulated message.</summary>
    [Obsolete("refer EncapsulatedMessageStructure instead")]
    public IImapBodyStructure BodyStructure => EncapsulatedMessageStructure;

    /// <summary>The body structure of encapsulated message.</summary>
    public IImapBodyStructure EncapsulatedMessageStructure { get; }

    public override bool HasEncapsulatedMessage => true;

    internal ImapMessageRfc822BodyStructure(string section,
                                            MimeType mediaType,
                                            IReadOnlyDictionary<string, string> parameters,
                                            string id,
                                            string description,
                                            string encoding,
                                            long size,
                                            ImapEnvelope envelope,
                                            IImapBodyStructure encapsulatedMessageStructure,
                                            long lineCount)
      : base(section: section,
             mediaType: mediaType,
             parameters: parameters,
             id: id,
             description: description,
             encoding: encoding,
             size: size,
             lineCount: lineCount)
    {
      Envelope                      = envelope ?? throw new ArgumentNullException(nameof(envelope));
      EncapsulatedMessageStructure  = encapsulatedMessageStructure ?? throw new ArgumentNullException(nameof(encapsulatedMessageStructure));

      ImapBodyStructureUtils.SetParentStructure(this);
    }

    private protected ImapMessageRfc822BodyStructure(ImapMessageRfc822BodyStructure baseStructure)
      : base(baseStructure)
    {
      if (baseStructure == null)
        throw new ArgumentNullException(nameof(baseStructure));

      Envelope                      = baseStructure.Envelope;
      EncapsulatedMessageStructure  = baseStructure.EncapsulatedMessageStructure;

      ImapBodyStructureUtils.SetParentStructure(this);
    }

    public override string ToString()
    {
      return StringConversion.ToString(GetType(), new (string, object)[] {
        (nameof(Section), Section),
        (nameof(MediaType), MediaType),
        (nameof(Parameters), Parameters),
        (nameof(Id), Id),
        (nameof(Description), Description),
        (nameof(Encoding), Encoding),
        (nameof(Size), Size),
        (nameof(LineCount), LineCount),
        (nameof(Envelope), Envelope),
        (nameof(EncapsulatedMessageStructure), EncapsulatedMessageStructure),
      });
    }
  }
}
