// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Net.Imap4.Urls;
using Smdn.Text;

namespace Smdn.Net.Imap4.Protocol.DataStructures.BodyStructures {
  public class ImapMultiPartBodyStructure : IImapBodyStructure {
    /*
     * IImapUrl impl
     */
    public Uri Url => ImapUriBuilder.BuildUri(uriBuilder);

    /// <summary>Infrastructure. It is not intended to be used directly from your code.</summary>
    void IImapUrl.SetBaseUrl(ImapUriBuilder baseUrl)
    {
      uriBuilder = baseUrl.Clone();
      uriBuilder.Section = Section;

      foreach (var nested in NestedPartStructures) {
        nested.SetBaseUrl(baseUrl);
      }
    }

    private ImapUriBuilder uriBuilder = null;

    /*
     * IImapBodyStructure impl
     */
    public IImapBodyStructure ParentStructure { get; internal set; }
    public string Section { get; }
    public MimeType MediaType { get; }
    public bool IsMultiPart => true;
    public bool HasEncapsulatedMessage => false;

    /*
     * nested structures
     */
    [Obsolete("refer NestedPartStructures instead")]
    public IReadOnlyList<IImapBodyStructure> NestedStructures => NestedPartStructures;
    public IReadOnlyList<IImapBodyStructure> NestedPartStructures { get; }

    internal ImapMultiPartBodyStructure(string section,
                                        IReadOnlyList<IImapBodyStructure> nestedPartStructures,
                                        string subtype)
    {
      Section               = section ?? throw new ArgumentNullException(nameof(section));
      MediaType             = MimeType.CreateMultipartType(subtype);
      NestedPartStructures  = nestedPartStructures ?? throw new ArgumentNullException(nameof(nestedPartStructures));

      ImapBodyStructureUtils.SetParentStructure(this);
    }

    private protected ImapMultiPartBodyStructure(ImapMultiPartBodyStructure baseStructure)
    {
      if (baseStructure == null)
        throw new ArgumentNullException(nameof(baseStructure));

      Section               = baseStructure.Section;
      MediaType             = baseStructure.MediaType;
      NestedPartStructures  = baseStructure.NestedPartStructures;

      ImapBodyStructureUtils.SetParentStructure(this);
    }

    public override string ToString()
    {
      return StringConversion.ToString(GetType(), new (string, object)[] {
        (nameof(Section), Section),
        (nameof(MediaType), MediaType),
        (nameof(NestedPartStructures), NestedPartStructures),
      });
    }
  }
}
