// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Net.Imap4.Protocol.DataStructures.BodyStructures;
using Smdn.Net.Imap4.Urls;
using Smdn.Net.MessageAccessProtocols.Strings;
using Smdn.Text;

namespace Smdn.Net.Imap4.Protocol.DataStructures.MessageAttributes {
  public sealed class ImapMessageAttributes :
    ImapMessageAttributesBase,
    IImapMessageDynamicAttributes,
    IImapMessageStaticAttributes,
    IImapUrl
  {
    /*
     * IImapMessageDynamicAttribute
     */
    private readonly ImapMessageDynamicAttributes dynamicAttributes;

    public IReadOnlyStringSet<ImapMessageFlag> Flags => dynamicAttributes.Flags;
    [CLSCompliant(false)]
    public ulong ModSeq => dynamicAttributes.ModSeq;

    /*
     * IImapMessageStaticAttribute
     */
    private readonly ImapMessageStaticAttributes staticAttributes;

    public IImapBodyStructure BodyStructure => staticAttributes.BodyStructure;
    public ImapEnvelope Envelope => staticAttributes.Envelope;
    public DateTimeOffset? InternalDate => staticAttributes.InternalDate;
    public long Rfc822Size => staticAttributes.Rfc822Size;
    public long Uid => staticAttributes.Uid;

    public IReadOnlyCollection<string> BinarySizeSpecifiers => staticAttributes.BinarySizeSpecifiers;
    public long GetFirstBinarySize() => staticAttributes.GetFirstBinarySize();
    public long GetBinarySizeOf(string section) => staticAttributes.GetBinarySizeOf(section);

    internal ImapMessageAttributes(long sequence)
      : base(sequence)
    {
      dynamicAttributes = new ImapMessageDynamicAttributes(sequence);
      staticAttributes = new ImapMessageStaticAttributes(sequence);
    }

    internal ImapMessageAttributes(ImapMessageFullAttributes fullAttributes)
      : base(fullAttributes.Sequence)
    {
      dynamicAttributes = fullAttributes.GetDynamicAttributes();
      staticAttributes = fullAttributes.GetStaticAttributes();
    }

    internal override ImapMessageDynamicAttributes GetDynamicAttributes() => dynamicAttributes;
    internal override ImapMessageStaticAttributes GetStaticAttributes() => staticAttributes;
    internal override ImapMessageBodyAttributes GetBodyAttributes() => null;

    public override string ToString()
    {
      return StringConversion.ToString(GetType(), new (string, object)[] {
        (nameof(Sequence), Sequence),
        (nameof(Uid), Uid),
        (nameof(BodyStructure), BodyStructure),
        (nameof(Envelope), Envelope),
        (nameof(InternalDate), InternalDate),
        (nameof(Rfc822Size), Rfc822Size),
        (nameof(BinarySizeSpecifiers), BinarySizeSpecifiers),
        (nameof(Flags), Flags),
        (nameof(ModSeq), ModSeq),
      });
    }

    /*
     * IImapUrl
     */
    public Uri Url => staticAttributes.Url;

    /// <summary>Infrastructure. It is not intended to be used directly from your code.</summary>
    void IImapUrl.SetBaseUrl(ImapUriBuilder baseUrl) => (staticAttributes as IImapUrl).SetBaseUrl(baseUrl);
  }
}
