// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Text;

namespace Smdn.Net.Imap4.Protocol.DataStructures {
  public sealed class ImapNamespace {
    public IReadOnlyList<ImapNamespaceDesc> PersonalNamespaces { get; }
    public IReadOnlyList<ImapNamespaceDesc> OtherUsersNamespaces { get; }
    public IReadOnlyList<ImapNamespaceDesc> SharedNamespaces { get; }

    internal ImapNamespace(IReadOnlyList<ImapNamespaceDesc> personalNamespaces = null,
                           IReadOnlyList<ImapNamespaceDesc> otherUsersNamespaces = null,
                           IReadOnlyList<ImapNamespaceDesc> sharedNamespaces = null)
    {
      PersonalNamespaces   = ToNonNull(personalNamespaces);
      OtherUsersNamespaces = ToNonNull(otherUsersNamespaces);
      SharedNamespaces     = ToNonNull(sharedNamespaces);

      IReadOnlyList<ImapNamespaceDesc> ToNonNull(IReadOnlyList<ImapNamespaceDesc> list)
      {
        return list
#if NET45
          ?? ArrayExtensions.Empty<ImapNamespaceDesc>();
#else
          ?? Array.Empty<ImapNamespaceDesc>();
#endif
      }
    }

    public override string ToString()
    {
      return StringConversion.ToString(GetType(), new (string, object)[] {
        (nameof(PersonalNamespaces), PersonalNamespaces),
        (nameof(OtherUsersNamespaces), OtherUsersNamespaces),
        (nameof(SharedNamespaces), SharedNamespaces),
      });
    }
  }
}
