// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;
using Smdn.Formats.ModifiedBase64;
using Smdn.Text;

namespace Smdn.Net.Imap4.Protocol.DataStructures {
  public sealed class ImapNamespaceDesc {
    public string Prefix { get; }
    public string HierarchyDelimiter { get; }
    public IReadOnlyDictionary<string, IReadOnlyList<string>> Extensions { get; }

    /*
     * RFC 5255 - Internet Message Access Protocol Internationalization
     * http://tools.ietf.org/html/rfc5255
     * 3.4. TRANSLATION Extension to the NAMESPACE Response
     */
    public string TranslatedPrefix {
      get
      {
        if (Extensions.TryGetValue("TRANSLATION", out var translatedPrefix))
          return ModifiedUTF7.Decode(translatedPrefix[0]);
        else
          return Prefix;
      }
    }

    internal ImapNamespaceDesc(string prefix,
                               string hierarchyDelimiter = null,
                               IReadOnlyDictionary<string, IReadOnlyList<string>> extensions = null)
    {
      Prefix = prefix ?? throw new ArgumentNullException(nameof(prefix));
      HierarchyDelimiter = hierarchyDelimiter;
      Extensions = extensions ?? ReadOnlyDictionary<string, IReadOnlyList<string>>.Empty;
    }

    public override string ToString()
    {
      return StringConversion.ToString(GetType(), new (string, object)[] {
        (nameof(Prefix), Prefix),
        (nameof(HierarchyDelimiter), HierarchyDelimiter),
        (nameof(Extensions), Extensions),
      });
    }
  }
}
