// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

namespace Smdn.Net.Imap4.Protocol.DataStructures {
  public static class ImapSequenceSet {
    public static IImapNumberSet All(bool asUidSet = false) => ImapNumberSet.All(asUidSet);
    public static IImapNumberSet Empty(bool asUidSet = false) => ImapNumberSet.Empty(asUidSet);
    public static IImapNumberSet From(long from, bool asUidSet = false) => ImapNumberSet.From(from, asUidSet);
    public static IImapNumberSet To(long to, bool asUidSet = false) => ImapNumberSet.To(to, asUidSet);
    public static IImapNumberSet Range(long from, long to, bool asUidSet = false) => ImapNumberSet.Range(from, to, asUidSet);
    public static IImapNumberSet Of(long number, bool asUidSet = false) => ImapNumberSet.Of(number, asUidSet);
    public static IImapNumberSet Of(long number, params long[] numbers) => ImapNumberSet.Of(asUidSet: false, number, numbers);
    public static IImapNumberSet Of(bool asUidSet, long number, params long[] numbers) => ImapNumberSet.Of(asUidSet: asUidSet, number, numbers);
    public static IImapNumberSet Of(IEnumerable<long> numbers, bool asUidSet = false) => ImapNumberSet.Of(numbers, asUidSet);
    public static IImapNumberSet Of(ArraySegment<long> numbers, bool asUidSet = false) => ImapNumberSet.Of(numbers, asUidSet);
#if false && NETSTANDARD2_1
    public static IImapSequenceSet Of(Span<long> numbers, bool asUidSet = false) => new NotImplementedException();
    public static IImapSequenceSet Of(Range numberRange, bool asUidSet = false) => new NotImplementedException();
#endif

    [Obsolete("use ImapUidSet.FromUri() instead", true)]
    public static IImapNumberSet FromUri(Uri uri) => throw new NotImplementedException("use ImapUidSet.FromUri() instead");

#region "create from:* set"
    [Obsolete("use From() instead", true)]
    public static IImapNumberSet CreateFromSet(long from) => throw new NotImplementedException("use From() instead");

    [Obsolete("use ImapUidSet.From() instead", true)]
    public static IImapNumberSet CreateUidFromSet(long from) => throw new NotImplementedException("use ImapUidSet.From() instead");

    [Obsolete("use ImapUidSet.From() or ImapSequenceSet.From() instead")]
    public static IImapNumberSet CreateFromSet(bool uidSet, long from) => throw new NotImplementedException("use ImapUidSet.From() or ImapSequenceSet.From() instead");
#endregion

#region "create *:to set"
    [Obsolete("use ImapUidSet.To()", true)]
    public static IImapNumberSet CreateToSet(long to) => throw new NotImplementedException("use To() instead");

    [Obsolete("use ImapUidSet.To() instead", true)]
    public static IImapNumberSet CreateUidToSet(long to) => throw new NotImplementedException("use ImapUidSet.To() instead");

    [Obsolete("use ImapUidSet.To() or ImapSequenceSet.To() instead")]
    public static IImapNumberSet CreateToSet(bool uidSet, long to) => throw new NotImplementedException("use ImapUidSet.To() or ImapSequenceSet.To() instead");
#endregion

#region "create from:to set"
    [Obsolete("use Range() instead", true)]
    public static IImapNumberSet CreateRangeSet(long from, long to) => throw new NotImplementedException("use Range() instead");

    [Obsolete("use ImapUidSet.Range() instead", true)]
    public static IImapNumberSet CreateUidRangeSet(long from, long to) => throw new NotImplementedException("use ImapUidSet.Range() instead");

    [Obsolete("use ImapUidSet.Range() or ImapSequenceSet.Range() instead")]
    public static IImapNumberSet CreateRangeSet(bool uidSet, long from, long to) => throw new NotImplementedException("use ImapUidSet.Range() or ImapSequenceSet.Range() instead");
#endregion

#region "create * set"
    [Obsolete("use All() instead", true)]
    public static IImapNumberSet CreateAllSet() => throw new NotImplementedException("use All() instead");

    [Obsolete("use ImapUidSet.All() instead", true)]
    public static IImapNumberSet CreateUidAllSet() => throw new NotImplementedException("use ImapUidSet.All() instead");

    [Obsolete("use ImapUidSet.All() or ImapSequenceSet.All() instead")]
    public static IImapNumberSet CreateAllSet(bool uid) => throw new NotImplementedException("use ImapUidSet.All() or ImapSequenceSet.All() instead");
#endregion

#region "create num*(,num) set"
    [Obsolete("use Of() instead", true)]
    public static IImapNumberSet CreateSet(long number, params long[] numbers) => throw new NotImplementedException("use Of() instead");

    [Obsolete("use ImapUidSet.Of() instead", true)]
    public static IImapNumberSet CreateUidSet(long number, params long[] numbers) => throw new NotImplementedException("use ImapUidSet.Of() instead");

    [Obsolete("use ImapUidSet.Of() or ImapSequenceSet.Of() instead", true)]
    public static IImapNumberSet CreateSet(bool uidSet, long number, params long[] numbers) => throw new NotImplementedException("use ImapUidSet.Of() or ImapSequenceSet.Of() instead");

    [Obsolete("use Of() instead", true)]
    public static IImapNumberSet CreateSet(long[] numbers) => throw new NotImplementedException("use Of() instead");

    [Obsolete("use ImapUidSet.Of() instead", true)]
    public static IImapNumberSet CreateUidSet(long[] numbers) => throw new NotImplementedException("use ImapUidSet.Of() instead");

    [Obsolete("use ImapUidSet.Of() or ImapSequenceSet.Of() instead", true)]
    public static IImapNumberSet CreateSet(bool uidSet, long[] numbers) => throw new NotImplementedException("use ImapUidSet.Of() or ImapSequenceSet.Of() instead");

    [Obsolete("use Of() instead", true)]
    public static IImapNumberSet CreateSet(IEnumerable<long> numbers) => throw new NotImplementedException("use Of() instead");

    [Obsolete("use ImapUidSet.Of() instead", true)]
    public static IImapNumberSet CreateUidSet(IEnumerable<long> numbers) => throw new NotImplementedException("use ImapUidSet.Of() instead");

    [Obsolete("use ImapUidSet.Of() or ImapSequenceSet.Of() instead", true)]
    public static IImapNumberSet CreateSet(bool uidSet, IEnumerable<long> numbers) => throw new NotImplementedException("use ImapUidSet.Of() or ImapSequenceSet.Of() instead");

    [Obsolete("use Of() instead", true)]
    public static IImapNumberSet CreateSet(long[] numbers, int start, int count) => throw new NotImplementedException("use Of() instead");

    [Obsolete("use ImapUidSet.Of() instead", true)]
    public static IImapNumberSet CreateUidSet(long[] numbers, int start, int count) => throw new NotImplementedException("use ImapUidSet.Of() instead");

    [Obsolete("use ImapUidSet.Of() or ImapSequenceSet.Of() instead", true)]
    public static IImapNumberSet CreateSet(bool uidSet, long[] numbers, int start, int count) => throw new NotImplementedException("use ImapUidSet.Of() or ImapSequenceSet.Of() instead");
#endregion
  }
}
