// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using Smdn.Net.Imap4.Protocol.Strings;

namespace Smdn.Net.Imap4.Protocol.Transmission.Transaction {
  internal sealed class RenameTransaction : ImapTransactionBase {
    public RenameTransaction(ImapConnection connection, IImapCommandTagGenerator commandTagGenerator)
      : base(connection, commandTagGenerator)
    {
    }

    // RFC 4466 - Collected Extensions to IMAP4 ABNF
    // http://tools.ietf.org/html/rfc4466
    // 2.3. Extended RENAME Command
    //    Arguments:  existing mailbox name
    //                new mailbox name
    //                OPTIONAL list of RENAME parameters
    //    Responses:  no specific responses for this command
    //    Result:     OK - rename completed
    //                NO - rename failure: cannot rename mailbox with
    //                     that name, cannot rename to mailbox with
    //                     that name, etc.
    //                BAD - argument(s) invalid
    protected override string GetCommand() => "RENAME";

    protected override ImapString[] GetCommandArguments()
    {
#if DEBUG
      if (!RequestArguments.ContainsKey("existing mailbox name") ||
          !RequestArguments.ContainsKey("new mailbox name"))
        throw CreateRequestArgumentsException("arguments 'existing mailbox name' and 'new mailbox name' must be setted");
#endif

      if (RequestArguments.TryGetValue("rename parameters", out var renameParameters))
        return new[] {
          RequestArguments["existing mailbox name"],
          RequestArguments["new mailbox name"],
          renameParameters,
        };
      else
        return new[] {
          RequestArguments["existing mailbox name"],
          RequestArguments["new mailbox name"],
        };
    }
  }
}
