// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using Smdn.Net.Imap4.Protocol.Session;
using Smdn.Net.Imap4.Protocol.Strings;

namespace Smdn.Net.Imap4.Protocol.Transmission.Transaction {
  internal sealed class StatusTransaction : ImapTransactionBase {
    public StatusTransaction(ImapConnection connection,
                             IImapCommandTagGenerator commandTagGenerator,
                             ImapMailboxManager mailboxManager)
      : base(connection, commandTagGenerator)
    {
      this.mailboxManager = mailboxManager;
    }

    // 6.3.10. STATUS Command
    //    Arguments:  mailbox name
    //                status data item names
    //    Responses:  untagged responses: STATUS
    //    Result:     OK - status completed
    //                NO - status failure: no status for that name
    //                BAD - command unknown or arguments invalid
    protected override string GetCommand() => "STATUS";

    protected override ImapString[] GetCommandArguments()
    {
#if DEBUG
      if (!RequestArguments.ContainsKey("mailbox name") ||
          !RequestArguments.ContainsKey("status data item names"))
        throw CreateRequestArgumentsException("arguments 'mailbox name' and 'status data item names' must be setted");
#endif

      return new[] {
        RequestArguments["mailbox name"],
        RequestArguments["status data item names"],
      };
    }

    protected override void OnDataResponseReceived(ImapServerDataResponse data)
    {
      if (data.Type == ImapServerDataResponseType.Status)
        ImapServerDataResponseParser.FromStatus(data, mailboxManager);

      base.OnDataResponseReceived(data);
    }

    private readonly ImapMailboxManager mailboxManager;
  }
}
