// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Net.Imap4.Protocol.Strings;
using Smdn.Net.MessageAccessProtocols.Strings;

namespace Smdn.Net.Imap4.Protocol {
  // combinable data item types:
  // * ImapCombinableDataItem
  //     => handles 'data item names' etc.
  //     ImapFetchDataItem
  //       => handles 'message data item names or macro'
  //     ImapSearchCriteria
  //       => handles 'searching criteria'
  //     ImapSortCriteria
  //       => handles 'sort criteria'
  //     ImapStatusDataItem
  //       => handles 'status data item names'
  //     ImapStoreDataItem
  //       => handles 'message data item name' and 'value for message data item'

  public abstract class ImapCombinableDataItems : ImapDataItems {
    private protected ImapCombinableDataItems(IReadOnlyCollection<ImapString> items)
      : base(items)
    {
    }

    private protected static IReadOnlyStringSet<ImapCapability> GetCombinedCapabilities(ImapCombinableDataItems x, ImapCombinableDataItems y)
    {
      if (x == null)
        throw new ArgumentNullException(nameof(x));
      if (y == null)
        throw new ArgumentNullException(nameof(y));

      ImapCapabilitySet requiredCapabilities = null;

      if (x is IImapExtension xe) {
        requiredCapabilities = new ImapCapabilitySet(caps: xe.RequiredCapabilities, asReadOnly: false);

        if (y is IImapExtension ye)
          requiredCapabilities.UnionWith(ye.RequiredCapabilities);
      }

      return requiredCapabilities ?? ImapCapabilitySet.ReadOnlyEmpty;
    }

    private protected static (IReadOnlyStringSet<ImapCapability> requiredCapabilities, IReadOnlyCollection<ImapString> items) GetCombined(ImapCombinableDataItems x, ImapCombinableDataItems y)
    {
      if (x == null)
        throw new ArgumentNullException(nameof(x));
      if (y == null)
        throw new ArgumentNullException(nameof(y));

      var items = new List<ImapString>(x.Items.Count + y.Items.Count);

      items.AddRange(x.Items);
      items.AddRange(y.Items);

      return (GetCombinedCapabilities(x, y), items);
    }
  }
}
