// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
#if SERIALIZATION
using System.Runtime.Serialization;
#endif

namespace Smdn.Net.Imap4.Protocol {
  // Exceptions:
  //   System.Excpetion
  //     System.SystemException
  //       Smdn.Net.Imap4.ImapExcetion
  //         Smdn.Net.Imap4.ImapInvalidOperationException
  //           Smdn.Net.Imap4.ImapIncapableException

#if SERIALIZATION
  [Serializable]
#endif
  public class ImapIncapableException : ImapInvalidOperationException {
    private readonly ImapCapability[] requiredCapabilities;

    public IReadOnlyList<ImapCapability> RequiredCapabilities => requiredCapabilities;

    public ImapIncapableException()
#if NET45
      : this(null, ArrayExtensions.Empty<ImapCapability>())
#else
      : this(null, Array.Empty<ImapCapability>())
#endif
    {
    }

    public ImapIncapableException(string message)
#if NET45
      : this(message, ArrayExtensions.Empty<ImapCapability>())
#else
      : this(message, Array.Empty<ImapCapability>())
#endif
    {
    }

    public ImapIncapableException(ImapCapability requiredCapability)
      : this(string.Concat(requiredCapability?.Value, " is incapable"), requiredCapability)
    {
    }

    public ImapIncapableException(IEnumerable<ImapCapability> requiredCapabilities)
      : this(string.Format("incapable ({0})", string.Join(", ", requiredCapabilities?.Select(capa => capa.Value))), requiredCapabilities)
    {
    }

    public ImapIncapableException(string message, ImapCapability requiredCapability)
      : this(message, new[] {requiredCapability})
    {
    }

    public ImapIncapableException(string message, IEnumerable<ImapCapability> requiredCapabilities)
      : this(message, requiredCapabilities.ToArray())
    {
    }

    private ImapIncapableException(string message, ImapCapability[] requiredCapabilities)
      : base(message)
    {
      this.requiredCapabilities = requiredCapabilities;
    }

#if SERIALIZATION
    protected ImapIncapableException(SerializationInfo info, StreamingContext context)
      : base(info, context)
    {
      this.requiredCapabilities = (ImapCapability[])info.GetValue("requiredCapabilities",
                                                                  typeof(ImapCapability[]));
    }

    public override void GetObjectData(SerializationInfo info, StreamingContext context)
    {
      base.GetObjectData(info, context);

      info.AddValue("requiredCapabilities", requiredCapabilities);
    }
#endif
  }
}
