// 
// Copyright (c) 2019 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

namespace Smdn.Net.Imap4.Protocol {
  public static class ImapMailboxNames {
    public const string Inbox = "INBOX";

    public static readonly IEqualityComparer<string> EqualityComparer = new MailboxNameEqualityComparer();

    private class MailboxNameEqualityComparer : EqualityComparer<string> {
      public override int GetHashCode(string obj) => IsInbox(obj) ? Inbox.GetHashCode() : StringComparer.Ordinal.GetHashCode(obj);

      public override bool Equals(string x, string y)
      {
        var isInboxX = IsInbox(x);
        var isInboxY = IsInbox(y);

        if (isInboxX && isInboxY)
          return true;
        if (isInboxX || isInboxY)
          return false;

        return string.Equals(x, y, StringComparison.Ordinal);
      }
    }

    public static bool IsInbox(string mailbox) => string.Equals(Inbox, mailbox, StringComparison.OrdinalIgnoreCase);

    internal static string GetValidated(string mailbox, string paramName)
    {
      if (mailbox == null)
        throw new ArgumentNullException(paramName ?? nameof(mailbox));
      if (mailbox.Length == 0)
        throw ExceptionUtils.CreateArgumentMustBeNonEmptyString(paramName ?? nameof(mailbox));

      return mailbox;
    }
  }
}
