using System;
using System.Text;
using System.IO;

using Smdn.IO.Streams.LineOriented;

namespace Smdn.Net.Imap4.Protocol.Transmission {
  internal class ImapPseudoResponseReceiver : IDisposable {
    public ImapPseudoResponseReceiver(string response)
    {
      baseStream = new MemoryStream(Encoding.ASCII.GetBytes(response), false);
      baseStream.Seek(0L, SeekOrigin.Begin);

      stream = new LineOrientedBufferedStream(baseStream);
      receiver = new ImapResponseReceiver(stream);
    }

    public ImapResponse ReceiveResponse()
    {
      if (receiver == null)
        throw new ObjectDisposedException(GetType().FullName);

      return receiver.ReceiveResponse();
    }

    public void Dispose()
    {
      baseStream?.Dispose();
      baseStream = null;

      receiver = null;
    }

    private ImapResponseReceiver receiver;
    private MemoryStream baseStream;
    private LineOrientedBufferedStream stream;
  }
}

