using System;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol.Transmission;

namespace Smdn.Net.Imap4.Protocol.DataStructures {
  [TestFixture]
  public class ImapQuotaResourceUsageTests {
    private static ImapQuotaResourceUsage FromResponse(string quotaResponse)
    {
      using (var receiver = new ImapPseudoResponseReceiver(quotaResponse)) {
        var response = receiver.ReceiveResponse() as ImapServerDataResponse;

        var quota = ImapServerDataResponseParser.FromQuota(response);

        return quota.Resources[0];
      }
    }

    [TestCase("STORAGE 10 512", 10.0 / 512.0)]
    [TestCase("STORAGE 0 512", 0.0)]
    [TestCase("STORAGE 10 1000", 10.0 / 1000.0)]
    [TestCase("STORAGE 999 1000", 999.0 / 1000.0)]
    [TestCase("STORAGE 1000 1000", 1.0)]
    public void TestUsageRatio(string quotaResource, double expectedUsage)
    {
      var quotaUsage = FromResponse($"* QUOTA \"root\" ({quotaResource})\r\n");

      Assert.AreEqual(expectedUsage, quotaUsage.UsageRatio);
    }

    [TestCase("STORAGE 0 0")]
    [TestCase("STORAGE 10 0")]
    [TestCase("STORAGE 100 0")]
    public void TestUsageRatio_LimitZero(string quotaResource)
    {
      var quotaUsage = FromResponse($"* QUOTA \"root\" ({quotaResource})\r\n");

      Assert.AreEqual(0L, quotaUsage.Limit);
      Assert.AreEqual(0.0, quotaUsage.UsageRatio);
    }

    [TestCase("STORAGE 10 512")]
    [TestCase("STORAGE 0 0")]
    [TestCase("STORAGE 100 0")]
    public void TestToString(string quotaResource)
    {
      var quotaUsage = FromResponse($"* QUOTA \"root\" ({quotaResource})\r\n");

      Assert.DoesNotThrow(() => quotaUsage.ToString());
    }
  }
}

