using System;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol.Strings;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapAuthenticationMechanismTests {
    [Test]
    public void TestEquals()
    {
      var x = ImapAuthenticationMechanism.CRAMMD5;

      Assert.IsTrue(x.Equals(ImapAuthenticationMechanism.CRAMMD5));
      Assert.IsTrue(x.Equals(new ImapAuthenticationMechanism("cram-md5")));
      Assert.IsTrue(x.Equals((object)"CRAM-MD5"));
      Assert.IsTrue(x.Equals((object)"cram-md5"));
      Assert.IsTrue(x.Equals((object)ImapAuthenticationMechanism.CRAMMD5));
      Assert.IsTrue(x.Equals("CRAM-MD5"));
      Assert.IsTrue(x.Equals("cram-md5"));
      Assert.IsTrue(x.Equals(ImapAuthenticationMechanism.CRAMMD5.ToString()));

      Assert.IsFalse(x.Equals(ImapAuthenticationMechanism.Login));
      Assert.IsFalse(x.Equals((ImapString)ImapAuthenticationMechanism.CRAMMD5.Value));
      Assert.IsFalse(x.Equals(new ImapCapability("CRAM-MD5")));
      Assert.IsFalse(x.Equals((object)new ImapCapability("CRAM-MD5")));
    }

    [Test]
    public void TestOpEquality()
    {
      var x = ImapAuthenticationMechanism.CRAMMD5;

      Assert.IsTrue(x == ImapAuthenticationMechanism.CRAMMD5);
      Assert.IsTrue(x == new ImapAuthenticationMechanism("cram-md5"));
      Assert.IsFalse(x != ImapAuthenticationMechanism.CRAMMD5);
      Assert.IsFalse(x != new ImapAuthenticationMechanism("cram-md5"));

      Assert.IsFalse(x == ImapAuthenticationMechanism.DigestMD5);
      Assert.IsFalse(x == new ImapAuthenticationMechanism("digest-md5"));
      Assert.IsTrue(x != ImapAuthenticationMechanism.DigestMD5);
      Assert.IsTrue(x != new ImapAuthenticationMechanism("digest-md5"));
    }

    [Test]
    public void TestToString()
    {
      Assert.AreEqual("CRAM-MD5", ImapAuthenticationMechanism.CRAMMD5.ToString());
    }

    [Test]
    public void TestSerializeBinary()
    {
      TestUtils.Assert.IsSerializableBinaryFormat(ImapAuthenticationMechanism.CRAMMD5, deserialized => {
        Assert.AreNotSame(ImapAuthenticationMechanism.CRAMMD5, deserialized);
        Assert.AreEqual(ImapAuthenticationMechanism.CRAMMD5, deserialized);
      });
    }
  }
}
