using System;
using System.Linq;
using NUnit.Framework;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapCapabilitySetExtensionsTests {
    [Test]
    public void TestIsCapable_CapabilitiesNull()
    {
      ImapPseudoCapabilitySet capabilities = null;

      Assert.Throws<ArgumentNullException>(() => capabilities.IsCapable(ImapCapability.Imap4Rev1));
    }

    [Test]
    public void TestIsCapable_EnumValueNull()
    {
      Assert.Throws<ArgumentNullException>(() => ImapCapability.AllCapabilities.IsCapable((ImapMailboxFlag)null));
    }

    [Test]
    public void TestIsCapable_DataItemNull()
    {
      Assert.Throws<ArgumentNullException>(() => ImapCapability.AllCapabilities.IsCapable((ImapStatusDataItems)null));
    }

    [Test]
    public void TestIsCapable_StandardFeatures()
    {
      Assert.IsTrue(ImapPseudoCapabilitySet.Empty.IsCapable(ImapFetchDataItems.All));
      Assert.IsTrue(ImapPseudoCapabilitySet.Empty.IsCapable(ImapFetchDataItems.BodyText()));
      Assert.IsTrue(ImapPseudoCapabilitySet.Empty.IsCapable(ImapStoreDataItems.AddFlags(ImapMessageFlag.Seen)));
      Assert.IsTrue(ImapPseudoCapabilitySet.Empty.IsCapable(ImapStatusDataItems.Unseen));

      Assert.IsTrue(ImapPseudoCapabilitySet.Empty.IsCapable(ImapMailboxFlag.NoInferiors));
      Assert.IsTrue(ImapPseudoCapabilitySet.Empty.IsCapable(ImapMessageFlag.Seen));
    }

    [Test]
    public void TestIsCapable_EnumValueWithExtension()
    {
      var feature = ImapMailboxFlag.HasChildren; // ImapCapability.ListExtended

      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(feature));

      var capabilitiesListExtended = new ImapPseudoCapabilitySet(ImapCapability.ListExtended);

      Assert.IsTrue(capabilitiesListExtended.IsCapable(feature));
      Assert.IsTrue(ImapCapability.AllCapabilities.IsCapable(feature));
    }

    [Test]
    public void TestIsCapable_DataItemWithExtension()
    {
      var feature = ImapStatusDataItems.HighestModSeq; // ImapCapability.CondStore

      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(feature));

      var capabilitiesListExtended = new ImapPseudoCapabilitySet(ImapCapability.CondStore);

      Assert.IsTrue(capabilitiesListExtended.IsCapable(feature));
      Assert.IsTrue(ImapCapability.AllCapabilities.IsCapable(feature));
    }
  }
}
