using System;
using System.IO;

using Smdn.IO; // StreamExtensions.Close
using Smdn.IO.Streams.LineOriented;
using Smdn.Net.Imap4.Protocol.Strings;
using Smdn.Net.Imap4.Protocol.Transmission;
using Smdn.Text.Encodings;

namespace Smdn.Net.Imap4.Protocol {
  public abstract class ImapCombinableDataItemsTestsBase {
    private class Sender : ImapSender {
      public bool SentAll => UnsentFragmentCount == 0;

      public Sender(LineOrientedBufferedStream stream)
        : base(stream)
      {
      }

      public void Enqueue(params ImapString[] strings) => Enqueue(true, strings);
      public void SendRemainder() => Send();
    }

    protected string Send(ImapCombinableDataItems item)
    {
      using (var stream = new MemoryStream()) {
        var sender = new Sender(new LineOrientedBufferedStream(stream));

        sender.Enqueue(item);

        while (!sender.SentAll) {
          sender.SendRemainder();
        }

        stream.Close();

        return OctetEncoding.EightBits.GetString(stream.ToArray()).TrimEnd(); // trim line terminator
      }
    }
  }
}
