using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapListReturnOptionsTests : ImapCombinableDataItemsTestsBase {
    [Test]
    public void TestCombine()
    {
      var o1 = ImapListReturnOptions.Children;
      var o2 = ImapListReturnOptions.Subscribed;

      Assert.AreEqual("(CHILDREN SUBSCRIBED)", Send(ImapListReturnOptions.Combine(o1, o2)));
      Assert.AreEqual("(CHILDREN SUBSCRIBED)", Send(o1 + o2));
      Assert.AreEqual("(CHILDREN SUBSCRIBED)", Send(o1.CombineWith(o2)));

      Assert.AreEqual("(SUBSCRIBED CHILDREN)", Send(ImapListReturnOptions.Combine(o2, o1)));
      Assert.AreEqual("(SUBSCRIBED CHILDREN)", Send(o2 + o1));
      Assert.AreEqual("(SUBSCRIBED CHILDREN)", Send(o2.CombineWith(o1)));
    }

    [Test]
    public void TestCombine_ContainsRequiredCapability()
    {
      var o1 = ImapListReturnOptions.Subscribed;
      var o2 = ImapListReturnOptions.SpecialUse;
      var capabilitiesListExtended = new ImapPseudoCapabilitySet(ImapCapability.ListExtended);
      var capabilitiesListExtendedSpecialUse = new ImapPseudoCapabilitySet(ImapCapability.ListExtended, ImapCapability.SpecialUse);

      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(o1));
      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(o2));
      Assert.IsTrue(capabilitiesListExtended.IsCapable(o1));
      Assert.IsFalse(capabilitiesListExtended.IsCapable(o2));
      Assert.IsTrue(capabilitiesListExtendedSpecialUse.IsCapable(o1));
      Assert.IsTrue(capabilitiesListExtendedSpecialUse.IsCapable(o2));

      foreach (var combined in new[] {
        ImapListReturnOptions.Combine(o1, o2),
        o1 + o2,
        o1.CombineWith(o2),
      }) {
        Assert.AreEqual("(SUBSCRIBED SPECIAL-USE)", Send(combined));

        Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(combined));
        Assert.IsFalse(capabilitiesListExtended.IsCapable(combined));
        Assert.IsTrue(capabilitiesListExtendedSpecialUse.IsCapable(combined));
      }
    }
  }
}
