using System;
using System.Collections.Generic;
using System.Linq;

using Smdn.Net.Imap4.Protocol.Strings;
using Smdn.Net.MessageAccessProtocols.Strings;

namespace Smdn.Net.Imap4.Protocol {
  internal class ImapPseudoCapabilitySet : ImapPseudoEnumValueSet<ImapCapability> {
    public static readonly ImapPseudoCapabilitySet Empty = new ImapPseudoCapabilitySet();

    public ImapPseudoCapabilitySet(params ImapCapability[] capabilities)
      : base(capabilities)
    {
    }
  }

  internal class ImapPseudoMessageFlagSet : ImapPseudoEnumValueSet<ImapMessageFlag> {
    public ImapPseudoMessageFlagSet()
      : base(Enumerable.Empty<ImapMessageFlag>())
    {
    }

    public ImapPseudoMessageFlagSet(IEnumerable<string> keywords, IEnumerable<ImapMessageFlag> flags)
      : base(flags.Concat(keywords.Select(keyword => new ImapMessageFlag(keyword))))
    {
    }
  }

  internal abstract class ImapPseudoEnumValueSet<TEnumValue> : HashSet<TEnumValue>, IReadOnlyStringSet<TEnumValue> where TEnumValue : ImapEnumValue {
    private class EnumValueEqualityComparer : EqualityComparer<TEnumValue> {
      public override bool Equals(TEnumValue x, TEnumValue y) => StringComparer.OrdinalIgnoreCase.Equals(x?.Value, y?.Value);
      public override int GetHashCode(TEnumValue obj) => StringComparer.OrdinalIgnoreCase.GetHashCode(obj?.Value);
    }

    protected ImapPseudoEnumValueSet(IEnumerable<TEnumValue> items)
      : base(items, new EnumValueEqualityComparer())
    {
    }

    public bool Contains(string value) => this.Any(i => Equals(value, i));
    public bool TryGetValue(string value, out TEnumValue item) => (item = this.FirstOrDefault(i => Equals(value, i))) != null;
    private bool Equals(string value, TEnumValue item) => StringComparer.OrdinalIgnoreCase.Equals(value, item?.Value);
  }
}
