using System;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol.Strings;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapServerDataResponseTypeTests {
    [Test]
    public void TestEquals()
    {
      var x = ImapServerDataResponseType.Fetch;

      Assert.IsTrue(x.Equals(ImapServerDataResponseType.Fetch));
      Assert.IsTrue(x.Equals((object)"FETCH"));
      Assert.IsTrue(x.Equals((object)"fetch"));
      Assert.IsTrue(x.Equals((object)ImapServerDataResponseType.Fetch));
      Assert.IsTrue(x.Equals("FETCH"));
      Assert.IsTrue(x.Equals("fetch"));
      Assert.IsTrue(x.Equals(ImapServerDataResponseType.Fetch.ToString()));

      Assert.IsFalse(x.Equals(ImapAuthenticationMechanism.Login));
      Assert.IsFalse(x.Equals((ImapString)ImapServerDataResponseType.Fetch.Value));
      Assert.IsFalse(x.Equals(new ImapCapability("FETCH")));
      Assert.IsFalse(x.Equals((object)new ImapCapability("FETCH")));
    }

    [Test]
    public void TestOpEquality()
    {
      var x = ImapServerDataResponseType.Fetch;

      Assert.IsTrue(x == ImapServerDataResponseType.Fetch);
      Assert.IsFalse(x != ImapServerDataResponseType.Fetch);

      Assert.IsFalse(x == ImapServerDataResponseType.Recent);
      Assert.IsTrue(x != ImapServerDataResponseType.Recent);
    }

    [Test]
    public void TestToString()
    {
      Assert.AreEqual("FETCH", ImapServerDataResponseType.Fetch.ToString());
      Assert.AreEqual("NAMESPACE", ImapServerDataResponseType.Namespace.ToString());
    }

    [Test]
    public void TestSerializeBinary()
    {
      TestUtils.Assert.IsSerializableBinaryFormat(ImapServerDataResponseType.Fetch, delegate(ImapServerDataResponseType deserialized) {
        Assert.AreNotSame(ImapServerDataResponseType.Fetch, deserialized);
        Assert.AreEqual(ImapServerDataResponseType.Fetch, deserialized);
      });
    }
  }
}
